/// <reference types="node" />
import * as stream from "stream";
import { IGetToken, IToken } from 'token-types';
export interface ITokenizer {
    /**
     * File length in bytes
     */
    fileSize?: number;
    readBuffer(buffer: Buffer, offset: number, length: number, position?: number): Promise<number>;
    readToken<T>(token: IGetToken<T>, position?: number | null): Promise<T>;
    readNumber(token: IGetToken<number>): Promise<number>;
    /**
     * Ignore given number of bytes
     * @param length Lenght in bytes
     */
    ignore(length: number): any;
}
export declare class IgnoreType implements IGetToken<Buffer> {
    len: number;
    constructor(len: number);
    get(buf: Buffer, off: number): Buffer;
}
export declare abstract class AbstractTokenizer implements ITokenizer {
    private numBuffer;
    abstract readBuffer(buffer: Buffer, offset: number, length: number, position: number): Promise<number>;
    private _readToken<T>(buffer, token);
    readToken<T>(token: IGetToken<T>, position?: number | null): Promise<T>;
    readNumber(token: IToken<number>): Promise<number>;
    abstract ignore(length: number): Promise<void>;
}
export declare class ReadStreamTokenizer extends AbstractTokenizer {
    private streamReader;
    constructor(stream: stream.Readable);
    readBuffer(buffer: Buffer, offset: number, length: number, position?: number): Promise<number>;
    static read(stream: stream.Readable): ReadStreamTokenizer;
    ignore(length: number): Promise<void>;
}
export declare class FileTokenizer extends AbstractTokenizer {
    private fd;
    fileSize: number;
    private fileOffset;
    private ignoreLength;
    constructor(fd: number, fileSize?: number);
    readBuffer(buffer: Buffer, offset: number, length: number, position?: number): Promise<number>;
    static open(filePath: string): Promise<FileTokenizer>;
    ignore(length: number): Promise<void>;
    close(): Promise<void>;
}
