"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require('assert');
var then_read_stream_1 = require("then-read-stream");
var fs = require("fs-extra");
/**
 * Used to reject read if end-of-stream has been reached
 * @type {Error}
 */
var EndOfFile = new Error("End-Of-File");
// Possibly call flush()
var maybeFlush = function (b, o, len, flush) {
    if (o + len > b.length) {
        if (typeof (flush) !== 'function') {
            throw new Error('Buffer out of space and no valid flush() function found');
        }
        flush(b, o);
        return 0;
    }
    return o;
};
var IgnoreType = (function () {
    function IgnoreType(len) {
        this.len = len;
    }
    // ToDo: don't read,, but skip data
    IgnoreType.prototype.get = function (buf, off) {
        return buf.slice(off, off + this.len);
    };
    return IgnoreType;
}());
exports.IgnoreType = IgnoreType;
var AbstractTokenizer = (function () {
    function AbstractTokenizer() {
        this.numBuffer = new Buffer(4);
    }
    AbstractTokenizer.prototype._readToken = function (buffer, token) {
        return this.readBuffer(buffer, 0, token.len, null).then(function (res) {
            return token.get(buffer, 0);
        });
    };
    AbstractTokenizer.prototype.readToken = function (token, position) {
        if (position === void 0) { position = null; }
        var buffer = new Buffer(token.len);
        return this.readBuffer(buffer, 0, token.len, position).then(function (res) {
            return token.get(buffer, 0);
        });
    };
    AbstractTokenizer.prototype.readNumber = function (token) {
        return this._readToken(this.numBuffer, token);
    };
    return AbstractTokenizer;
}());
exports.AbstractTokenizer = AbstractTokenizer;
var ReadStreamTokenizer = (function (_super) {
    __extends(ReadStreamTokenizer, _super);
    function ReadStreamTokenizer(stream) {
        var _this = _super.call(this) || this;
        _this.streamReader = new then_read_stream_1.StreamReader(stream);
        return _this;
    }
    ReadStreamTokenizer.prototype.readBuffer = function (buffer, offset, length, position) {
        if (position === void 0) { position = null; }
        return this.streamReader.read(buffer, offset, length, position); // ToDo: looks like wrong return type is defined in fs.read
    };
    ReadStreamTokenizer.read = function (stream) {
        return new ReadStreamTokenizer(stream);
    };
    ReadStreamTokenizer.prototype.ignore = function (length) {
        var buf = new Buffer(length);
        return this.streamReader.read(buf, 0, length); // stream cannot skip data
    };
    return ReadStreamTokenizer;
}(AbstractTokenizer));
exports.ReadStreamTokenizer = ReadStreamTokenizer;
var FileTokenizer = (function (_super) {
    __extends(FileTokenizer, _super);
    function FileTokenizer(fd, fileSize) {
        var _this = _super.call(this) || this;
        _this.fd = fd;
        _this.fileSize = fileSize;
        _this.fileOffset = 0;
        _this.ignoreLength = 0;
        return _this;
    }
    FileTokenizer.prototype.readBuffer = function (buffer, offset, length, position) {
        var _this = this;
        if (position === void 0) { position = null; }
        if (position) {
            this.fileOffset = position;
            this.ignoreLength = 0;
        }
        else if (this.ignoreLength > 0) {
            position = this.fileOffset + this.ignoreLength;
            this.ignoreLength = 0;
        }
        return fs.read(this.fd, buffer, offset, length, position).then(function (bytesRead) {
            _this.fileOffset += bytesRead;
        }); // ToDo: looks like wrong return type is defined in fs.read
    };
    FileTokenizer.open = function (filePath) {
        return fs.pathExists(filePath).then(function (exist) {
            if (!exist) {
                throw new Error("File not found: " + filePath);
            }
            return fs.stat(filePath).then(function (stat) {
                return fs.open(filePath, "r").then(function (fd) {
                    return new FileTokenizer(fd, stat.size);
                });
            });
        });
    };
    FileTokenizer.prototype.ignore = function (length) {
        this.ignoreLength += length;
        return Promise.resolve(null);
    };
    FileTokenizer.prototype.close = function () {
        return fs.close(this.fd);
    };
    return FileTokenizer;
}(AbstractTokenizer));
exports.FileTokenizer = FileTokenizer;
